#!/usr/bin/env python
import os
import sys
import pytest


def main():
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "project.settings.testing")
    sys.path.insert(0, "examples/simple")
    sys.path.insert(0, "src")
    return pytest.main()


if __name__ == "__main__":
    sys.exit(main())



import operator
from typing import Dict, Union, List, Set, Tuple

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("Registry",)


class Registry(type):
    REGISTRY = {}  # type: Dict[str, Registry]

    def __new__(mcs, name, bases, attrs):
        new_cls = type.__new__(mcs, name, bases, attrs)
        # Here the name of the class is used as key but it could be any class
        # parameter.
        if getattr(new_cls, "_uid", None):
            mcs.REGISTRY[new_cls._uid] = new_cls
        if new_cls.__name__ != "BaseCurrency":
            new_cls.validate()
        return new_cls

    @property
    def _uid(cls) -> str:
        return getattr(cls, "uid", cls.__name__)

    @classmethod
    def reset(mcs) -> None:
        mcs.REGISTRY = {}

    @classmethod
    def get(mcs, key, default=None):
        return mcs.REGISTRY.get(key, default)

    @classmethod
    def items(mcs):
        return mcs.REGISTRY.items()

    @classmethod
    def values(
        mcs,
        limit_choices_to: Union[Tuple[str, ...], List[str], Set[str]] = None,
        sort_by_key: bool = False,
    ):
        if limit_choices_to is None:
            values = [
                (__key, __value.name)
                for __key, __value in mcs.REGISTRY.items()
            ]
        else:
            values = [
                (__key, __value.name)
                for __key, __value in mcs.REGISTRY.items()
                if __key in limit_choices_to
            ]
        if sort_by_key:
            values.sort(key=operator.itemgetter(0))
        else:
            values.sort(key=operator.itemgetter(1))
        return values

    @classmethod
    def values_with_code(
        mcs,
        limit_choices_to: Union[Tuple[str, ...], List[str], Set[str]] = None,
        sort_by_key: bool = False,
    ):
        if limit_choices_to is None:
            values = [
                (__key, f"{__value.name} ({__key})")
                for __key, __value in mcs.REGISTRY.items()
            ]
        else:
            values = [
                (__key, f"{__value.name} ({__key})")
                for __key, __value in mcs.REGISTRY.items()
                if __key in limit_choices_to
            ]
        if sort_by_key:
            values.sort(key=operator.itemgetter(0))
        else:
            values.sort(key=operator.itemgetter(1))
        return values



import os
import sys

import valuta

path = os.path.join(os.path.abspath(os.path.dirname(__name__)), 'benchmarks')

sys.path.insert(0, path)

from constants import CURRENCIES, TEST_CYCLES

try:
    if callable(profile):
        pass
except:
    from fallbacks import profile


@profile
def main():
    for _ in range(TEST_CYCLES):
        for currency_cls in CURRENCIES:
            currency_cls.convert_to_currency_units(1_000)


if __name__ == '__main__':
    main()



from decimal import Decimal
from typing import Optional, Union

from babel.numbers import get_currency_name, get_currency_symbol

from .exceptions import ImproperlyConfigured
from .helpers import classproperty
from .registry import Registry

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("BaseCurrency",)


class BaseCurrency(metaclass=Registry):
    """Base currency.

    Each currency typically has a main currency unit (the dollar, for
    example, or the euro) and a fractional unit, often defined as 1⁄100 of
    the main unit: 100 cents = 1 dollar, 100 centimes = 1 franc,
    100 pence = 1 pound, although units of 1⁄10 or 1⁄1000 occasionally also
    occur. Some currencies do not have any smaller units at all, such as the
    Icelandic krona.

    https://en.wikipedia.org/wiki/Currency
    """

    uid: Optional[str] = None
    name: str
    rate: int

    @classmethod
    def validate(cls):
        """Constructor."""
        if not hasattr(cls, "rate"):
            raise ImproperlyConfigured(
                "The `rate` property of the currency shall be defined."
            )

    @classmethod
    def convert_to_currency_units(
        cls, value: Union[int, float, Decimal]
    ) -> Union[int, float, Decimal]:
        """Convert to amount in currency units."""
        return value / cls.rate

    @classproperty
    def name(cls):
        """Automatic currency name.

        Example::

            from babel.numbers import get_currency_name
            In: get_currency_name('AMD')
            Out: 'Armenian Dram'

            In: get_currency_name('AMD', locale='hy_AM')
            Out: 'հայկական դրամ'

            In: get_currency_name('AMD', locale='nl_NL')
            Out: 'Armeense dram'

        :return:
        """
        return get_currency_name(cls.uid)

    @classproperty
    def symbol(cls):
        """Automatic currency symbol.

        Example::

            from babel.numbers import get_currency_symbol
            In: get_currency_symbol('AMD')
            Out: 'AMD'

            In: get_currency_symbol('AMD', locale='hy_AM')
            Out: '֏'

            In: get_currency_symbol('AMD', locale='nl_NL')
            Out: 'AMD'

        :return:
        """
        return get_currency_symbol(cls.uid)



from copy import deepcopy
import logging
import unittest

from ..utils import get_currency_choices_with_code
from ..registry import Registry

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("TestRegistry",)

LOGGER = logging.getLogger(__name__)


class TestRegistry(unittest.TestCase):
    """Test registry."""

    def setUp(self) -> None:
        self.registry_backup = deepcopy(Registry.REGISTRY)

    def tearDown(self) -> None:
        Registry.REGISTRY = self.registry_backup

    def test_get_currency_choices(self):
        """Test Registry.reset."""
        Registry.reset()
        choices = get_currency_choices_with_code()
        self.assertListEqual(choices, [])

    def test_registry_items(self):
        items = Registry.items()
        choices = get_currency_choices_with_code()
        self.assertEqual(len(items), len(choices))



from ..base import BaseCurrency

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("DJF",)


class DJF(BaseCurrency):
    """DJF - Djiboutian franc."""

    uid: str = "DJF"
    rate: int = 100



# Generated by Django 3.2.2 on 2021-05-15 20:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='price_with_tax',
            field=models.IntegerField(default=1),
            preserve_default=False,
        ),
    ]



import os

__all__ = (
    "project_dir",
    "gettext",
    "PROJECT_DIR",
)


def project_dir(base):
    return os.path.abspath(
        os.path.join(os.path.dirname(__file__), base).replace("\\", "/")
    )


def gettext(s):
    return s


PROJECT_DIR = project_dir



# Generated by Django 3.2.3 on 2021-05-17 19:12

from django.db import migrations, models
import valuta.contrib.django_integration.models


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0002_product_price_with_tax'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProductProxyCastToDecimal',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('price', models.IntegerField()),
                ('price_with_tax', models.IntegerField()),
                ('currency', valuta.contrib.django_integration.models.CurrencyField(choices=[('AFN', 'Afghan Afghani'), ('ALL', 'Albanian Lek'), ('DZD', 'Algerian Dinar'), ('AOA', 'Angolan Kwanza'), ('ARS', 'Argentine Peso'), ('AMD', 'Armenian Dram'), ('AWG', 'Aruban Florin'), ('AUD', 'Australian Dollar'), ('AZN', 'Azerbaijani Manat'), ('BSD', 'Bahamian Dollar'), ('BHD', 'Bahraini Dinar'), ('BDT', 'Bangladeshi Taka'), ('BBD', 'Barbadian Dollar'), ('BYN', 'Belarusian Ruble'), ('BZD', 'Belize Dollar'), ('BMD', 'Bermudan Dollar'), ('BTN', 'Bhutanese Ngultrum'), ('BTC', 'Bitcoin'), ('BOB', 'Bolivian Boliviano'), ('BAM', 'Bosnia-Herzegovina Convertible Mark'), ('BWP', 'Botswanan Pula'), ('BRL', 'Brazilian Real'), ('GBP', 'British Pound'), ('BND', 'Brunei Dollar'), ('BGN', 'Bulgarian Lev'), ('BIF', 'Burundian Franc'), ('XPF', 'CFP Franc'), ('CKD', 'CKD'), ('KHR', 'Cambodian Riel'), ('CAD', 'Canadian Dollar'), ('CVE', 'Cape Verdean Escudo'), ('KYD', 'Cayman Islands Dollar'), ('XAF', 'Central African CFA Franc'), ('CLP', 'Chilean Peso'), ('CNY', 'Chinese Yuan'), ('COP', 'Colombian Peso'), ('KMF', 'Comorian Franc'), ('CDF', 'Congolese Franc'), ('CRC', 'Costa Rican Colón'), ('HRK', 'Croatian Kuna'), ('CUP', 'Cuban Peso'), ('CZK', 'Czech Koruna'), ('DKK', 'Danish Krone'), ('DJF', 'Djiboutian Franc'), ('DOP', 'Dominican Peso'), ('XCD', 'East Caribbean Dollar'), ('EGP', 'Egyptian Pound'), ('ERN', 'Eritrean Nakfa'), ('ETB', 'Ethiopian Birr'), ('EUR', 'Euro'), ('FOK', 'FOK'), ('FKP', 'Falkland Islands Pound'), ('FJD', 'Fijian Dollar'), ('GGP', 'GGP'), ('GMD', 'Gambian Dalasi'), ('GEL', 'Georgian Lari'), ('GHS', 'Ghanaian Cedi'), ('GIP', 'Gibraltar Pound'), ('GTQ', 'Guatemalan Quetzal'), ('GNF', 'Guinean Franc'), ('GYD', 'Guyanaese Dollar'), ('HTG', 'Haitian Gourde'), ('HNL', 'Honduran Lempira'), ('HKD', 'Hong Kong Dollar'), ('HUF', 'Hungarian Forint'), ('IMP', 'IMP'), ('ISK', 'Icelandic Króna'), ('INR', 'Indian Rupee'), ('IDR', 'Indonesian Rupiah'), ('IRR', 'Iranian Rial'), ('IQD', 'Iraqi Dinar'), ('ILS', 'Israeli New Shekel'), ('JEP', 'JEP'), ('JMD', 'Jamaican Dollar'), ('JPY', 'Japanese Yen'), ('JOD', 'Jordanian Dinar'), ('KID', 'KID'), ('KZT', 'Kazakhstani Tenge'), ('KES', 'Kenyan Shilling'), ('KWD', 'Kuwaiti Dinar'), ('KGS', 'Kyrgystani Som'), ('LAK', 'Laotian Kip'), ('LBP', 'Lebanese Pound'), ('LSL', 'Lesotho Loti'), ('LRD', 'Liberian Dollar'), ('LYD', 'Libyan Dinar'), ('MOP', 'Macanese Pataca'), ('MKD', 'Macedonian Denar'), ('MGA', 'Malagasy Ariary'), ('MWK', 'Malawian Kwacha'), ('MYR', 'Malaysian Ringgit'), ('MVR', 'Maldivian Rufiyaa'), ('MRU', 'Mauritanian Ouguiya'), ('MUR', 'Mauritian Rupee'), ('MXN', 'Mexican Peso'), ('MDL', 'Moldovan Leu'), ('MNT', 'Mongolian Tugrik'), ('MAD', 'Moroccan Dirham'), ('MZN', 'Mozambican Metical'), ('MMK', 'Myanmar Kyat'), ('NAD', 'Namibian Dollar'), ('NPR', 'Nepalese Rupee'), ('ANG', 'Netherlands Antillean Guilder'), ('TWD', 'New Taiwan Dollar'), ('NZD', 'New Zealand Dollar'), ('NIO', 'Nicaraguan Córdoba'), ('NGN', 'Nigerian Naira'), ('KPW', 'North Korean Won'), ('NOK', 'Norwegian Krone'), ('OMR', 'Omani Rial'), ('PND', 'PND'), ('PRB', 'PRB'), ('PKR', 'Pakistani Rupee'), ('PAB', 'Panamanian Balboa'), ('PGK', 'Papua New Guinean Kina'), ('PYG', 'Paraguayan Guarani'), ('PEN', 'Peruvian Sol'), ('PHP', 'Philippine Piso'), ('PLN', 'Polish Zloty'), ('QAR', 'Qatari Rial'), ('RON', 'Romanian Leu'), ('RUB', 'Russian Ruble'), ('RWF', 'Rwandan Franc'), ('SLS', 'SLS'), ('WST', 'Samoan Tala'), ('SAR', 'Saudi Riyal'), ('RSD', 'Serbian Dinar'), ('SCR', 'Seychellois Rupee'), ('SLL', 'Sierra Leonean Leone'), ('SGD', 'Singapore Dollar'), ('SBD', 'Solomon Islands Dollar'), ('SOS', 'Somali Shilling'), ('ZAR', 'South African Rand'), ('KRW', 'South Korean Won'), ('SSP', 'South Sudanese Pound'), ('LKR', 'Sri Lankan Rupee'), ('SHP', 'St. Helena Pound'), ('SDG', 'Sudanese Pound'), ('SRD', 'Surinamese Dollar'), ('SZL', 'Swazi Lilangeni'), ('SEK', 'Swedish Krona'), ('CHF', 'Swiss Franc'), ('SYP', 'Syrian Pound'), ('STN', 'São Tomé & Príncipe Dobra'), ('TVD', 'TVD'), ('TJS', 'Tajikistani Somoni'), ('TZS', 'Tanzanian Shilling'), ('THB', 'Thai Baht'), ('TOP', 'Tongan Paʻanga'), ('TTD', 'Trinidad & Tobago Dollar'), ('TND', 'Tunisian Dinar'), ('TRY', 'Turkish Lira'), ('TMT', 'Turkmenistani Manat'), ('USD', 'US Dollar'), ('UGX', 'Ugandan Shilling'), ('UAH', 'Ukrainian Hryvnia'), ('AED', 'United Arab Emirates Dirham'), ('UYU', 'Uruguayan Peso'), ('UZS', 'Uzbekistani Som'), ('VUV', 'Vanuatu Vatu'), ('VES', 'Venezuelan Bolívar'), ('VND', 'Vietnamese Dong'), ('XOF', 'West African CFA Franc'), ('YER', 'Yemeni Rial'), ('ZWB', 'ZWB'), ('ZMW', 'Zambian Kwacha')], max_length=10)),
            ],
            options={
                'verbose_name': 'Product proxy (cast_to=decimal)',
                'verbose_name_plural': 'Product proxies (cast_to=decimal)',
                'db_table': 'product_product',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='ProductProxyCastToFloat',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('price', models.IntegerField()),
                ('price_with_tax', models.IntegerField()),
                ('currency', valuta.contrib.django_integration.models.CurrencyField(choices=[('AFN', 'Afghan Afghani'), ('ALL', 'Albanian Lek'), ('DZD', 'Algerian Dinar'), ('AOA', 'Angolan Kwanza'), ('ARS', 'Argentine Peso'), ('AMD', 'Armenian Dram'), ('AWG', 'Aruban Florin'), ('AUD', 'Australian Dollar'), ('AZN', 'Azerbaijani Manat'), ('BSD', 'Bahamian Dollar'), ('BHD', 'Bahraini Dinar'), ('BDT', 'Bangladeshi Taka'), ('BBD', 'Barbadian Dollar'), ('BYN', 'Belarusian Ruble'), ('BZD', 'Belize Dollar'), ('BMD', 'Bermudan Dollar'), ('BTN', 'Bhutanese Ngultrum'), ('BTC', 'Bitcoin'), ('BOB', 'Bolivian Boliviano'), ('BAM', 'Bosnia-Herzegovina Convertible Mark'), ('BWP', 'Botswanan Pula'), ('BRL', 'Brazilian Real'), ('GBP', 'British Pound'), ('BND', 'Brunei Dollar'), ('BGN', 'Bulgarian Lev'), ('BIF', 'Burundian Franc'), ('XPF', 'CFP Franc'), ('CKD', 'CKD'), ('KHR', 'Cambodian Riel'), ('CAD', 'Canadian Dollar'), ('CVE', 'Cape Verdean Escudo'), ('KYD', 'Cayman Islands Dollar'), ('XAF', 'Central African CFA Franc'), ('CLP', 'Chilean Peso'), ('CNY', 'Chinese Yuan'), ('COP', 'Colombian Peso'), ('KMF', 'Comorian Franc'), ('CDF', 'Congolese Franc'), ('CRC', 'Costa Rican Colón'), ('HRK', 'Croatian Kuna'), ('CUP', 'Cuban Peso'), ('CZK', 'Czech Koruna'), ('DKK', 'Danish Krone'), ('DJF', 'Djiboutian Franc'), ('DOP', 'Dominican Peso'), ('XCD', 'East Caribbean Dollar'), ('EGP', 'Egyptian Pound'), ('ERN', 'Eritrean Nakfa'), ('ETB', 'Ethiopian Birr'), ('EUR', 'Euro'), ('FOK', 'FOK'), ('FKP', 'Falkland Islands Pound'), ('FJD', 'Fijian Dollar'), ('GGP', 'GGP'), ('GMD', 'Gambian Dalasi'), ('GEL', 'Georgian Lari'), ('GHS', 'Ghanaian Cedi'), ('GIP', 'Gibraltar Pound'), ('GTQ', 'Guatemalan Quetzal'), ('GNF', 'Guinean Franc'), ('GYD', 'Guyanaese Dollar'), ('HTG', 'Haitian Gourde'), ('HNL', 'Honduran Lempira'), ('HKD', 'Hong Kong Dollar'), ('HUF', 'Hungarian Forint'), ('IMP', 'IMP'), ('ISK', 'Icelandic Króna'), ('INR', 'Indian Rupee'), ('IDR', 'Indonesian Rupiah'), ('IRR', 'Iranian Rial'), ('IQD', 'Iraqi Dinar'), ('ILS', 'Israeli New Shekel'), ('JEP', 'JEP'), ('JMD', 'Jamaican Dollar'), ('JPY', 'Japanese Yen'), ('JOD', 'Jordanian Dinar'), ('KID', 'KID'), ('KZT', 'Kazakhstani Tenge'), ('KES', 'Kenyan Shilling'), ('KWD', 'Kuwaiti Dinar'), ('KGS', 'Kyrgystani Som'), ('LAK', 'Laotian Kip'), ('LBP', 'Lebanese Pound'), ('LSL', 'Lesotho Loti'), ('LRD', 'Liberian Dollar'), ('LYD', 'Libyan Dinar'), ('MOP', 'Macanese Pataca'), ('MKD', 'Macedonian Denar'), ('MGA', 'Malagasy Ariary'), ('MWK', 'Malawian Kwacha'), ('MYR', 'Malaysian Ringgit'), ('MVR', 'Maldivian Rufiyaa'), ('MRU', 'Mauritanian Ouguiya'), ('MUR', 'Mauritian Rupee'), ('MXN', 'Mexican Peso'), ('MDL', 'Moldovan Leu'), ('MNT', 'Mongolian Tugrik'), ('MAD', 'Moroccan Dirham'), ('MZN', 'Mozambican Metical'), ('MMK', 'Myanmar Kyat'), ('NAD', 'Namibian Dollar'), ('NPR', 'Nepalese Rupee'), ('ANG', 'Netherlands Antillean Guilder'), ('TWD', 'New Taiwan Dollar'), ('NZD', 'New Zealand Dollar'), ('NIO', 'Nicaraguan Córdoba'), ('NGN', 'Nigerian Naira'), ('KPW', 'North Korean Won'), ('NOK', 'Norwegian Krone'), ('OMR', 'Omani Rial'), ('PND', 'PND'), ('PRB', 'PRB'), ('PKR', 'Pakistani Rupee'), ('PAB', 'Panamanian Balboa'), ('PGK', 'Papua New Guinean Kina'), ('PYG', 'Paraguayan Guarani'), ('PEN', 'Peruvian Sol'), ('PHP', 'Philippine Piso'), ('PLN', 'Polish Zloty'), ('QAR', 'Qatari Rial'), ('RON', 'Romanian Leu'), ('RUB', 'Russian Ruble'), ('RWF', 'Rwandan Franc'), ('SLS', 'SLS'), ('WST', 'Samoan Tala'), ('SAR', 'Saudi Riyal'), ('RSD', 'Serbian Dinar'), ('SCR', 'Seychellois Rupee'), ('SLL', 'Sierra Leonean Leone'), ('SGD', 'Singapore Dollar'), ('SBD', 'Solomon Islands Dollar'), ('SOS', 'Somali Shilling'), ('ZAR', 'South African Rand'), ('KRW', 'South Korean Won'), ('SSP', 'South Sudanese Pound'), ('LKR', 'Sri Lankan Rupee'), ('SHP', 'St. Helena Pound'), ('SDG', 'Sudanese Pound'), ('SRD', 'Surinamese Dollar'), ('SZL', 'Swazi Lilangeni'), ('SEK', 'Swedish Krona'), ('CHF', 'Swiss Franc'), ('SYP', 'Syrian Pound'), ('STN', 'São Tomé & Príncipe Dobra'), ('TVD', 'TVD'), ('TJS', 'Tajikistani Somoni'), ('TZS', 'Tanzanian Shilling'), ('THB', 'Thai Baht'), ('TOP', 'Tongan Paʻanga'), ('TTD', 'Trinidad & Tobago Dollar'), ('TND', 'Tunisian Dinar'), ('TRY', 'Turkish Lira'), ('TMT', 'Turkmenistani Manat'), ('USD', 'US Dollar'), ('UGX', 'Ugandan Shilling'), ('UAH', 'Ukrainian Hryvnia'), ('AED', 'United Arab Emirates Dirham'), ('UYU', 'Uruguayan Peso'), ('UZS', 'Uzbekistani Som'), ('VUV', 'Vanuatu Vatu'), ('VES', 'Venezuelan Bolívar'), ('VND', 'Vietnamese Dong'), ('XOF', 'West African CFA Franc'), ('YER', 'Yemeni Rial'), ('ZWB', 'ZWB'), ('ZMW', 'Zambian Kwacha')], max_length=10)),
            ],
            options={
                'verbose_name': 'Product proxy (cast_to=float)',
                'verbose_name_plural': 'Product proxies (cast_to=float)',
                'db_table': 'product_product',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='ProductProxyCastToInt',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('price', models.IntegerField()),
                ('price_with_tax', models.IntegerField()),
                ('currency', valuta.contrib.django_integration.models.CurrencyField(choices=[('AFN', 'Afghan Afghani'), ('ALL', 'Albanian Lek'), ('DZD', 'Algerian Dinar'), ('AOA', 'Angolan Kwanza'), ('ARS', 'Argentine Peso'), ('AMD', 'Armenian Dram'), ('AWG', 'Aruban Florin'), ('AUD', 'Australian Dollar'), ('AZN', 'Azerbaijani Manat'), ('BSD', 'Bahamian Dollar'), ('BHD', 'Bahraini Dinar'), ('BDT', 'Bangladeshi Taka'), ('BBD', 'Barbadian Dollar'), ('BYN', 'Belarusian Ruble'), ('BZD', 'Belize Dollar'), ('BMD', 'Bermudan Dollar'), ('BTN', 'Bhutanese Ngultrum'), ('BTC', 'Bitcoin'), ('BOB', 'Bolivian Boliviano'), ('BAM', 'Bosnia-Herzegovina Convertible Mark'), ('BWP', 'Botswanan Pula'), ('BRL', 'Brazilian Real'), ('GBP', 'British Pound'), ('BND', 'Brunei Dollar'), ('BGN', 'Bulgarian Lev'), ('BIF', 'Burundian Franc'), ('XPF', 'CFP Franc'), ('CKD', 'CKD'), ('KHR', 'Cambodian Riel'), ('CAD', 'Canadian Dollar'), ('CVE', 'Cape Verdean Escudo'), ('KYD', 'Cayman Islands Dollar'), ('XAF', 'Central African CFA Franc'), ('CLP', 'Chilean Peso'), ('CNY', 'Chinese Yuan'), ('COP', 'Colombian Peso'), ('KMF', 'Comorian Franc'), ('CDF', 'Congolese Franc'), ('CRC', 'Costa Rican Colón'), ('HRK', 'Croatian Kuna'), ('CUP', 'Cuban Peso'), ('CZK', 'Czech Koruna'), ('DKK', 'Danish Krone'), ('DJF', 'Djiboutian Franc'), ('DOP', 'Dominican Peso'), ('XCD', 'East Caribbean Dollar'), ('EGP', 'Egyptian Pound'), ('ERN', 'Eritrean Nakfa'), ('ETB', 'Ethiopian Birr'), ('EUR', 'Euro'), ('FOK', 'FOK'), ('FKP', 'Falkland Islands Pound'), ('FJD', 'Fijian Dollar'), ('GGP', 'GGP'), ('GMD', 'Gambian Dalasi'), ('GEL', 'Georgian Lari'), ('GHS', 'Ghanaian Cedi'), ('GIP', 'Gibraltar Pound'), ('GTQ', 'Guatemalan Quetzal'), ('GNF', 'Guinean Franc'), ('GYD', 'Guyanaese Dollar'), ('HTG', 'Haitian Gourde'), ('HNL', 'Honduran Lempira'), ('HKD', 'Hong Kong Dollar'), ('HUF', 'Hungarian Forint'), ('IMP', 'IMP'), ('ISK', 'Icelandic Króna'), ('INR', 'Indian Rupee'), ('IDR', 'Indonesian Rupiah'), ('IRR', 'Iranian Rial'), ('IQD', 'Iraqi Dinar'), ('ILS', 'Israeli New Shekel'), ('JEP', 'JEP'), ('JMD', 'Jamaican Dollar'), ('JPY', 'Japanese Yen'), ('JOD', 'Jordanian Dinar'), ('KID', 'KID'), ('KZT', 'Kazakhstani Tenge'), ('KES', 'Kenyan Shilling'), ('KWD', 'Kuwaiti Dinar'), ('KGS', 'Kyrgystani Som'), ('LAK', 'Laotian Kip'), ('LBP', 'Lebanese Pound'), ('LSL', 'Lesotho Loti'), ('LRD', 'Liberian Dollar'), ('LYD', 'Libyan Dinar'), ('MOP', 'Macanese Pataca'), ('MKD', 'Macedonian Denar'), ('MGA', 'Malagasy Ariary'), ('MWK', 'Malawian Kwacha'), ('MYR', 'Malaysian Ringgit'), ('MVR', 'Maldivian Rufiyaa'), ('MRU', 'Mauritanian Ouguiya'), ('MUR', 'Mauritian Rupee'), ('MXN', 'Mexican Peso'), ('MDL', 'Moldovan Leu'), ('MNT', 'Mongolian Tugrik'), ('MAD', 'Moroccan Dirham'), ('MZN', 'Mozambican Metical'), ('MMK', 'Myanmar Kyat'), ('NAD', 'Namibian Dollar'), ('NPR', 'Nepalese Rupee'), ('ANG', 'Netherlands Antillean Guilder'), ('TWD', 'New Taiwan Dollar'), ('NZD', 'New Zealand Dollar'), ('NIO', 'Nicaraguan Córdoba'), ('NGN', 'Nigerian Naira'), ('KPW', 'North Korean Won'), ('NOK', 'Norwegian Krone'), ('OMR', 'Omani Rial'), ('PND', 'PND'), ('PRB', 'PRB'), ('PKR', 'Pakistani Rupee'), ('PAB', 'Panamanian Balboa'), ('PGK', 'Papua New Guinean Kina'), ('PYG', 'Paraguayan Guarani'), ('PEN', 'Peruvian Sol'), ('PHP', 'Philippine Piso'), ('PLN', 'Polish Zloty'), ('QAR', 'Qatari Rial'), ('RON', 'Romanian Leu'), ('RUB', 'Russian Ruble'), ('RWF', 'Rwandan Franc'), ('SLS', 'SLS'), ('WST', 'Samoan Tala'), ('SAR', 'Saudi Riyal'), ('RSD', 'Serbian Dinar'), ('SCR', 'Seychellois Rupee'), ('SLL', 'Sierra Leonean Leone'), ('SGD', 'Singapore Dollar'), ('SBD', 'Solomon Islands Dollar'), ('SOS', 'Somali Shilling'), ('ZAR', 'South African Rand'), ('KRW', 'South Korean Won'), ('SSP', 'South Sudanese Pound'), ('LKR', 'Sri Lankan Rupee'), ('SHP', 'St. Helena Pound'), ('SDG', 'Sudanese Pound'), ('SRD', 'Surinamese Dollar'), ('SZL', 'Swazi Lilangeni'), ('SEK', 'Swedish Krona'), ('CHF', 'Swiss Franc'), ('SYP', 'Syrian Pound'), ('STN', 'São Tomé & Príncipe Dobra'), ('TVD', 'TVD'), ('TJS', 'Tajikistani Somoni'), ('TZS', 'Tanzanian Shilling'), ('THB', 'Thai Baht'), ('TOP', 'Tongan Paʻanga'), ('TTD', 'Trinidad & Tobago Dollar'), ('TND', 'Tunisian Dinar'), ('TRY', 'Turkish Lira'), ('TMT', 'Turkmenistani Manat'), ('USD', 'US Dollar'), ('UGX', 'Ugandan Shilling'), ('UAH', 'Ukrainian Hryvnia'), ('AED', 'United Arab Emirates Dirham'), ('UYU', 'Uruguayan Peso'), ('UZS', 'Uzbekistani Som'), ('VUV', 'Vanuatu Vatu'), ('VES', 'Venezuelan Bolívar'), ('VND', 'Vietnamese Dong'), ('XOF', 'West African CFA Franc'), ('YER', 'Yemeni Rial'), ('ZWB', 'ZWB'), ('ZMW', 'Zambian Kwacha')], max_length=10)),
            ],
            options={
                'verbose_name': 'Product proxy (cast_to=int)',
                'verbose_name_plural': 'Product proxies (cast_to=int)',
                'db_table': 'product_product',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='ProductProxyLimitChoicesTo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('price', models.IntegerField()),
                ('price_with_tax', models.IntegerField()),
                ('currency', valuta.contrib.django_integration.models.CurrencyField(choices=[('AFN', 'Afghan Afghani'), ('ALL', 'Albanian Lek'), ('DZD', 'Algerian Dinar'), ('AOA', 'Angolan Kwanza'), ('ARS', 'Argentine Peso'), ('AMD', 'Armenian Dram'), ('AWG', 'Aruban Florin'), ('AUD', 'Australian Dollar'), ('AZN', 'Azerbaijani Manat'), ('BSD', 'Bahamian Dollar'), ('BHD', 'Bahraini Dinar'), ('BDT', 'Bangladeshi Taka'), ('BBD', 'Barbadian Dollar'), ('BYN', 'Belarusian Ruble'), ('BZD', 'Belize Dollar'), ('BMD', 'Bermudan Dollar'), ('BTN', 'Bhutanese Ngultrum'), ('BTC', 'Bitcoin'), ('BOB', 'Bolivian Boliviano'), ('BAM', 'Bosnia-Herzegovina Convertible Mark'), ('BWP', 'Botswanan Pula'), ('BRL', 'Brazilian Real'), ('GBP', 'British Pound'), ('BND', 'Brunei Dollar'), ('BGN', 'Bulgarian Lev'), ('BIF', 'Burundian Franc'), ('XPF', 'CFP Franc'), ('CKD', 'CKD'), ('KHR', 'Cambodian Riel'), ('CAD', 'Canadian Dollar'), ('CVE', 'Cape Verdean Escudo'), ('KYD', 'Cayman Islands Dollar'), ('XAF', 'Central African CFA Franc'), ('CLP', 'Chilean Peso'), ('CNY', 'Chinese Yuan'), ('COP', 'Colombian Peso'), ('KMF', 'Comorian Franc'), ('CDF', 'Congolese Franc'), ('CRC', 'Costa Rican Colón'), ('HRK', 'Croatian Kuna'), ('CUP', 'Cuban Peso'), ('CZK', 'Czech Koruna'), ('DKK', 'Danish Krone'), ('DJF', 'Djiboutian Franc'), ('DOP', 'Dominican Peso'), ('XCD', 'East Caribbean Dollar'), ('EGP', 'Egyptian Pound'), ('ERN', 'Eritrean Nakfa'), ('ETB', 'Ethiopian Birr'), ('EUR', 'Euro'), ('FOK', 'FOK'), ('FKP', 'Falkland Islands Pound'), ('FJD', 'Fijian Dollar'), ('GGP', 'GGP'), ('GMD', 'Gambian Dalasi'), ('GEL', 'Georgian Lari'), ('GHS', 'Ghanaian Cedi'), ('GIP', 'Gibraltar Pound'), ('GTQ', 'Guatemalan Quetzal'), ('GNF', 'Guinean Franc'), ('GYD', 'Guyanaese Dollar'), ('HTG', 'Haitian Gourde'), ('HNL', 'Honduran Lempira'), ('HKD', 'Hong Kong Dollar'), ('HUF', 'Hungarian Forint'), ('IMP', 'IMP'), ('ISK', 'Icelandic Króna'), ('INR', 'Indian Rupee'), ('IDR', 'Indonesian Rupiah'), ('IRR', 'Iranian Rial'), ('IQD', 'Iraqi Dinar'), ('ILS', 'Israeli New Shekel'), ('JEP', 'JEP'), ('JMD', 'Jamaican Dollar'), ('JPY', 'Japanese Yen'), ('JOD', 'Jordanian Dinar'), ('KID', 'KID'), ('KZT', 'Kazakhstani Tenge'), ('KES', 'Kenyan Shilling'), ('KWD', 'Kuwaiti Dinar'), ('KGS', 'Kyrgystani Som'), ('LAK', 'Laotian Kip'), ('LBP', 'Lebanese Pound'), ('LSL', 'Lesotho Loti'), ('LRD', 'Liberian Dollar'), ('LYD', 'Libyan Dinar'), ('MOP', 'Macanese Pataca'), ('MKD', 'Macedonian Denar'), ('MGA', 'Malagasy Ariary'), ('MWK', 'Malawian Kwacha'), ('MYR', 'Malaysian Ringgit'), ('MVR', 'Maldivian Rufiyaa'), ('MRU', 'Mauritanian Ouguiya'), ('MUR', 'Mauritian Rupee'), ('MXN', 'Mexican Peso'), ('MDL', 'Moldovan Leu'), ('MNT', 'Mongolian Tugrik'), ('MAD', 'Moroccan Dirham'), ('MZN', 'Mozambican Metical'), ('MMK', 'Myanmar Kyat'), ('NAD', 'Namibian Dollar'), ('NPR', 'Nepalese Rupee'), ('ANG', 'Netherlands Antillean Guilder'), ('TWD', 'New Taiwan Dollar'), ('NZD', 'New Zealand Dollar'), ('NIO', 'Nicaraguan Córdoba'), ('NGN', 'Nigerian Naira'), ('KPW', 'North Korean Won'), ('NOK', 'Norwegian Krone'), ('OMR', 'Omani Rial'), ('PND', 'PND'), ('PRB', 'PRB'), ('PKR', 'Pakistani Rupee'), ('PAB', 'Panamanian Balboa'), ('PGK', 'Papua New Guinean Kina'), ('PYG', 'Paraguayan Guarani'), ('PEN', 'Peruvian Sol'), ('PHP', 'Philippine Piso'), ('PLN', 'Polish Zloty'), ('QAR', 'Qatari Rial'), ('RON', 'Romanian Leu'), ('RUB', 'Russian Ruble'), ('RWF', 'Rwandan Franc'), ('SLS', 'SLS'), ('WST', 'Samoan Tala'), ('SAR', 'Saudi Riyal'), ('RSD', 'Serbian Dinar'), ('SCR', 'Seychellois Rupee'), ('SLL', 'Sierra Leonean Leone'), ('SGD', 'Singapore Dollar'), ('SBD', 'Solomon Islands Dollar'), ('SOS', 'Somali Shilling'), ('ZAR', 'South African Rand'), ('KRW', 'South Korean Won'), ('SSP', 'South Sudanese Pound'), ('LKR', 'Sri Lankan Rupee'), ('SHP', 'St. Helena Pound'), ('SDG', 'Sudanese Pound'), ('SRD', 'Surinamese Dollar'), ('SZL', 'Swazi Lilangeni'), ('SEK', 'Swedish Krona'), ('CHF', 'Swiss Franc'), ('SYP', 'Syrian Pound'), ('STN', 'São Tomé & Príncipe Dobra'), ('TVD', 'TVD'), ('TJS', 'Tajikistani Somoni'), ('TZS', 'Tanzanian Shilling'), ('THB', 'Thai Baht'), ('TOP', 'Tongan Paʻanga'), ('TTD', 'Trinidad & Tobago Dollar'), ('TND', 'Tunisian Dinar'), ('TRY', 'Turkish Lira'), ('TMT', 'Turkmenistani Manat'), ('USD', 'US Dollar'), ('UGX', 'Ugandan Shilling'), ('UAH', 'Ukrainian Hryvnia'), ('AED', 'United Arab Emirates Dirham'), ('UYU', 'Uruguayan Peso'), ('UZS', 'Uzbekistani Som'), ('VUV', 'Vanuatu Vatu'), ('VES', 'Venezuelan Bolívar'), ('VND', 'Vietnamese Dong'), ('XOF', 'West African CFA Franc'), ('YER', 'Yemeni Rial'), ('ZWB', 'ZWB'), ('ZMW', 'Zambian Kwacha')], max_length=10)),
            ],
            options={
                'verbose_name': 'Product proxy (limit_choices_to)',
                'verbose_name_plural': 'Product proxies (limit_choices_to)',
                'db_table': 'product_product',
                'managed': False,
            },
        ),
        migrations.AlterModelOptions(
            name='product',
            options={'verbose_name': 'Product', 'verbose_name_plural': 'Products'},
        ),
    ]



import logging
import subprocess
import tempfile
import unittest

from .data import (
    LIST_CURRENCIES_OUTPUT,
    LIST_CURRENCIES_OUTPUT_SORTED_BY_VALUE,
)

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("TestCLI",)

LOGGER = logging.getLogger(__name__)


class TestCLI(unittest.TestCase):
    """CLI tests."""

    def test_generate_currencies(self):
        """Test generate currencies CLI."""
        out_dir = tempfile.mkdtemp()
        res = subprocess.check_output(
            [
                "valuta-generate-currencies",
                "--out-dir",
                out_dir,
                "--skip-first-line",
            ]
        ).strip()
        self.assertEqual(res, b"")
        return res

    def test_list_currencies(self):
        """Test list currencies CLI."""
        res = subprocess.check_output(
            [
                "valuta-list-currencies",
            ]
        ).strip()
        self.assertEqual(res, LIST_CURRENCIES_OUTPUT)
        return res

    def test_list_currencies_sort_by_value(self):
        """Test list currencies CLI, sorted byvalue."""
        res = subprocess.check_output(
            [
                "valuta-list-currencies",
                "--sort-by-value",
            ]
        ).strip()
        self.assertEqual(res, LIST_CURRENCIES_OUTPUT_SORTED_BY_VALUE)
        return res



import operator
from typing import List, Tuple, Set, Union

from babel.numbers import get_currency_symbol
from valuta.registry import Registry

__all__ = ("get_currency_choices_with_sign",)


def get_currency_choices_with_sign(
    limit_choices_to: Union[Tuple[str, ...], List[str], Set[str]] = None,
    sort_by_key: bool = False,
) -> List[Tuple[str, str]]:
    """Get currency choices with code.

    List of choices in the following format::

        [
            ("AMD", "AMD - Armenian Dram"),
            ("EUR", "€ - Euro"),
            ("USD", "$ - US Dollar"),
        ]
    """
    if limit_choices_to is None:
        values = [
            (__key, f"{get_currency_symbol(__key)} - {__value.name}")
            for __key, __value in Registry.REGISTRY.items()
        ]
    else:
        values = [
            (__key, f"{get_currency_symbol(__key)} - {__value.name}")
            for __key, __value in Registry.REGISTRY.items()
            if __key in limit_choices_to
        ]
    if sort_by_key:
        values.sort(key=operator.itemgetter(0))
    else:
        values.sort(key=operator.itemgetter(1))
    return values



from .base import *

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "filters": {
        "require_debug_false": {"()": "django.utils.log.RequireDebugFalse"}
    },
    "root": {
        "level": "INFO",
        "handlers": ["all_log"],
    },
    "formatters": {
        "verbose": {
            "format": "\n%(levelname)s %(asctime)s [%(pathname)s:%(lineno)s] %(message)s"
        },
        "simple": {"format": "\n%(levelname)s %(message)s"},
    },
    "handlers": {
        "mail_admins": {
            "level": "ERROR",
            "filters": ["require_debug_false"],
            "class": "django.utils.log.AdminEmailHandler",
        },
        "console": {
            "level": "ERROR",
            "class": "logging.StreamHandler",
            "formatter": "verbose",
        },
        "all_log": {
            "level": "ERROR",
            "class": "logging.handlers.RotatingFileHandler",
            "filename": PROJECT_DIR("../../../../var/logs/all.log"),
            "maxBytes": 1048576,
            "backupCount": 99,
            "formatter": "verbose",
        },
        "django_log": {
            "level": "ERROR",
            "class": "logging.handlers.RotatingFileHandler",
            "filename": PROJECT_DIR("../../../../var/logs/django.log"),
            "maxBytes": 1048576,
            "backupCount": 99,
            "formatter": "verbose",
        },
        "django_request_log": {
            "level": "ERROR",
            "class": "logging.handlers.RotatingFileHandler",
            "filename": PROJECT_DIR("../../../../var/logs/django_request.log"),
            "maxBytes": 1048576,
            "backupCount": 99,
            "formatter": "verbose",
        },
        "valuta_log": {
            "level": "ERROR",
            "class": "logging.handlers.RotatingFileHandler",
            "filename": PROJECT_DIR("../../../../var/logs/valuta.log"),
            "maxBytes": 1048576,
            "backupCount": 99,
            "formatter": "verbose",
        },
    },
    "loggers": {
        "django.request": {
            "handlers": ["django_request_log"],
            "level": "INFO",
            "propagate": True,
        },
        "django": {
            "handlers": ["django_log"],
            "level": "ERROR",
            "propagate": False,
        },
        "valuta": {
            "handlers": ["console", "valuta_log"],
            "level": "ERROR",
            "propagate": True,
        },
    },
}



from factory import LazyAttribute, Faker
from factory.django import DjangoModelFactory
from factory.fuzzy import FuzzyChoice

from valuta.utils import get_currency_choices

from product.models import Product

__all__ = ("ProductFactory",)


class BaseProductFactory(DjangoModelFactory):
    """Base product factory."""

    name = Faker("text", max_nb_chars=100)
    price = Faker("pyint", min_value=100, max_value=1_000_000)
    price_with_tax = LazyAttribute(lambda __x: int(__x.price * 1.20))
    currency = FuzzyChoice([key for key, value in get_currency_choices()])

    class Meta:
        """Meta class."""

        model = Product
        abstract = True


class ProductFactory(BaseProductFactory):
    """Product factory."""



# Generated by Django 3.2.2 on 2021-05-11 23:45

from django.db import migrations, models
import valuta.contrib.django_integration.models
import valuta.currencies.amd
import valuta.currencies.eur


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('price', models.IntegerField()),
                ('currency', valuta.contrib.django_integration.models.CurrencyField(choices=[('AMD', valuta.currencies.amd.AMD), ('EUR', valuta.currencies.eur.EUR)], max_length=10)),
            ],
        ),
    ]



from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from .models import (
    Product,
    ProductProxyLimitChoicesTo,
    ProductProxyCastToInt,
    ProductProxyCastToFloat,
    ProductProxyCastToDecimal,
)

__all__ = (
    "ProductAdmin",
    "ProductProxyLimitChoicesToAdmin",
    "ProductProxyCastToIntAdmin",
    "ProductProxyCastToFloatAdmin",
    "ProductProxyCastToDecimalAdmin",
)


class ProductAdminMixin:
    """Product admin mixin."""

    list_display = (
        "name",
        "price",
        "price_with_tax",
        "currency",
        "display_price_in_currency_units",
        "display_price_with_tax_in_currency_units",
    )

    def display_price_in_currency_units(self, obj):
        return obj.price_in_currency_units()

    display_price_in_currency_units.short_description = _(
        "Price in currency units"
    )
    display_price_in_currency_units.admin_order_field = "price"

    def display_price_with_tax_in_currency_units(self, obj):
        return obj.price_with_tax_in_currency_units()

    display_price_with_tax_in_currency_units.short_description = _(
        "Price with tax in currency units"
    )
    display_price_with_tax_in_currency_units.admin_order_field = (
        "price_with_tax"
    )


@admin.register(Product)
class ProductAdmin(admin.ModelAdmin, ProductAdminMixin):
    """Product admin."""

    list_display = ProductAdminMixin.list_display


@admin.register(ProductProxyLimitChoicesTo)
class ProductProxyLimitChoicesToAdmin(admin.ModelAdmin, ProductAdminMixin):
    """ProductProxyLimitChoicesTo admin."""

    list_display = ProductAdminMixin.list_display


@admin.register(ProductProxyCastToInt)
class ProductProxyCastToIntAdmin(admin.ModelAdmin, ProductAdminMixin):
    """ProductProxyCastToInt admin."""

    list_display = ProductAdminMixin.list_display


@admin.register(ProductProxyCastToFloat)
class ProductProxyCastToFloatAdmin(admin.ModelAdmin, ProductAdminMixin):
    """ProductProxyCastToFloat admin."""

    list_display = ProductAdminMixin.list_display


@admin.register(ProductProxyCastToDecimal)
class ProductProxyCastToDecimalAdmin(admin.ModelAdmin, ProductAdminMixin):
    """ProductProxyCastToDecimal admin."""

    list_display = ProductAdminMixin.list_display



from functools import partialmethod
from typing import Union, List, Tuple, Optional, Set, Callable

from django.db import models
from django.db.models import (
    IntegerField,
    BigIntegerField,
    SmallIntegerField,
    FloatField,
)
from django.conf import settings

from ...registry import Registry
from ...utils import get_currency_choices_with_code

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("CurrencyField",)


class CurrencyField(models.CharField):
    """Currency field."""

    def __init__(
        self,
        *args,
        amount_fields: Optional[
            Union[List[str], Tuple[str, ...], Set[str]]
        ] = None,
        limit_choices_to: Optional[
            Union[List[str], Tuple[str, ...], Set[str]]
        ] = None,
        get_choices_func: Callable = get_currency_choices_with_code,
        cast_to: Optional[Callable] = None,
        **kwargs,
    ):
        self.amount_fields = amount_fields
        self.cast_to = cast_to
        if get_choices_func is None:
            get_choices_func = get_currency_choices_with_code
        kwargs["max_length"] = 10
        if limit_choices_to is None:
            settings_limit_choices_to = getattr(
                settings, "VALUTA_FIELD_LIMIT_CHOICES_TO", None
            )
            if settings_limit_choices_to:
                limit_choices_to = settings_limit_choices_to

        kwargs["choices"] = get_choices_func(limit_choices_to)
        super().__init__(*args, **kwargs)

    def contribute_to_class(self, cls, name, **kwargs):
        super().contribute_to_class(cls, name, **kwargs)
        if not self.null:
            setattr(
                cls,
                f"get_currency_cls_for_{self.name}",
                partialmethod(self.__class__._get_currency_cls, field=self),
            )
            if self.amount_fields:
                for amount_field in self.amount_fields:
                    setattr(
                        cls,
                        f"{amount_field}_in_currency_units",
                        partialmethod(
                            self.__class__._convert_to_currency_units,
                            field=self,
                            amount_field=amount_field,
                        ),
                    )

    def _get_currency_cls(
        self,
        field: Union[
            IntegerField, BigIntegerField, SmallIntegerField, FloatField
        ],
        **kwargs,
    ):
        key = getattr(self, field.name)
        if not key:
            return None
        return Registry.get(key)

    def _convert_to_currency_units(
        self,
        field: Union[
            IntegerField, BigIntegerField, SmallIntegerField, FloatField
        ],
        amount_field: str,
        **kwargs,
    ):
        key = getattr(self, field.name)
        if not key:
            return None
        currency_cls = Registry.get(key)
        if currency_cls is None:
            return None
        amount_in_fractional_units = getattr(self, amount_field)
        value = currency_cls.convert_to_currency_units(
            amount_in_fractional_units
        )
        if field.cast_to:
            return field.cast_to(value)
        return value



from decimal import Decimal

from django.db import models
from django.utils.translation import gettext_lazy as _

import valuta
from valuta.utils import get_currency_choices, get_currency_choices_with_code
from valuta.contrib.django_integration.models import CurrencyField

from .helpers import get_currency_choices_with_sign

__all__ = (
    "AbstractProduct",
    "Product",
    "ProductProxyCastToInt",
    "ProductProxyCastToFloat",
    "ProductProxyCastToDecimal",
    "ProductProxyLimitChoicesTo",
    "ProductProxyChoicesFuncNone",
)


class AbstractProduct(models.Model):
    """Abstract product model."""

    name = models.CharField(_("Name"), max_length=255)
    price = models.IntegerField(_("Price"))
    price_with_tax = models.IntegerField(_("Price with tax"))

    class Meta:
        abstract = True
        verbose_name = _("Product")
        verbose_name_plural = _("Products")

    def __str__(self):
        return self.name


class Product(AbstractProduct):
    """Product model."""

    currency = CurrencyField(
        _("Currency"),
        amount_fields=(
            "price",
            "price_with_tax",
        ),
        get_choices_func=get_currency_choices_with_code,
    )

    class Meta:
        verbose_name = _("Product")
        verbose_name_plural = _("Products")


class ProductProxyCastToInt(AbstractProduct):
    """A sort of a proxy model to Product.

    Casts amount in fractional units to int.
    """

    currency = CurrencyField(
        _("Currency"),
        amount_fields=(
            "price",
            "price_with_tax",
        ),
        cast_to=int,
    )

    class Meta:
        managed = False
        db_table = Product._meta.db_table
        verbose_name = _("Product proxy (cast_to=int)")
        verbose_name_plural = _("Product proxies (cast_to=int)")


class ProductProxyCastToFloat(AbstractProduct):
    """A sort of a proxy model to Product.

    Casts amount in fractional units to float.
    """

    currency = CurrencyField(
        _("Currency"),
        amount_fields=(
            "price",
            "price_with_tax",
        ),
        get_choices_func=get_currency_choices,
        cast_to=float,
    )

    class Meta:
        managed = False
        db_table = Product._meta.db_table
        verbose_name = _("Product proxy (cast_to=float)")
        verbose_name_plural = _("Product proxies (cast_to=float)")


class ProductProxyCastToDecimal(AbstractProduct):
    """A sort of a proxy model to Product.

    Casts amount in fractional units to decimal.
    """

    currency = CurrencyField(
        _("Currency"),
        amount_fields=(
            "price",
            "price_with_tax",
        ),
        get_choices_func=get_currency_choices_with_sign,
        cast_to=lambda __v: Decimal(str(__v)),
    )

    class Meta:
        managed = False
        db_table = Product._meta.db_table
        verbose_name = _("Product proxy (cast_to=decimal)")
        verbose_name_plural = _("Product proxies (cast_to=decimal)")


class ProductProxyLimitChoicesTo(AbstractProduct):
    """A sort of a proxy model to Product.

    Casts amount in fractional units to int.
    """

    currency = CurrencyField(
        _("Currency"),
        amount_fields=(
            "price",
            "price_with_tax",
        ),
        limit_choices_to=(
            valuta.EUR.uid,
            valuta.AMD.uid,
        ),
        get_choices_func=get_currency_choices_with_code,
    )

    class Meta:
        managed = False
        db_table = Product._meta.db_table
        verbose_name = _("Product proxy (limit_choices_to)")
        verbose_name_plural = _("Product proxies (limit_choices_to)")


class ProductProxyChoicesFuncNone(AbstractProduct):
    """A sort of a proxy model to Product.

    The `get_choices_func` argument is None here.
    """

    currency = CurrencyField(
        _("Currency"),
        amount_fields=(
            "price",
            "price_with_tax",
        ),
        limit_choices_to=(
            valuta.EUR.uid,
            valuta.AMD.uid,
        ),
        get_choices_func=None,
    )

    class Meta:
        managed = False
        db_table = Product._meta.db_table
        verbose_name = _("Product proxy (get_choices_func=None)")
        verbose_name_plural = _("Product proxies (get_choices_func=None)")



from django.conf.urls import re_path
from django.contrib import admin
from django.views.generic import TemplateView

__all__ = ("urlpatterns",)

urlpatterns = [
    re_path(
        "^$", view=TemplateView.as_view(template_name="home.html"), name="home"
    ),
    re_path(r"^admin/", admin.site.urls),
]
